/*
** GmurfProjectSaver.c
** To save a Gmurf project. Seems to work well.
**
** sam-lo_p@epita.fr
*/

#include"GmurfProjectSaver.h"

// copies and compresses wave files through the zlib.

char* remove_all_slashes(char* str)
{
  char* tmpstr;
  int i;

  i = strlen(str)-1;
  while ((i >=0) && (str[i] != '/'))
    i--;
  if (strcmp(str+i,"")!=0)
    {
      tmpstr = (char*) malloc(sizeof(char)*(i+2));
      strcpy(tmpstr, str+i+1);
    }
  else
    {
      tmpstr = (char*) malloc(sizeof(char)*(strlen(str)));
      strcpy(tmpstr, str);
    }
  return(tmpstr);
}

int project_copy_files(main_mixer, dirname)
     t_main_mixer* main_mixer;
     char*         dirname;

{
  char         buf[CPY_BUF_SIZE];
  char         filename[MAX_LINE_LEN];
  char         filedirname[MAX_LINE_LEN];
  char*        tmpstr;
  t_mixer_elt* mixer_elt;
  int          result;
  int          srcfd;
  gzFile*       dstfd;


  strcpy(filename, "aaa");
  strcat(filename, GMURF_WAVE_EXTENSION);
  mixer_elt = main_mixer->first_mixer_elt;
  while (mixer_elt != NULL)
    {
      if ((srcfd = open(mixer_elt->tempgmurf->tempname, O_RDONLY)) < 0)
	{
	  warning("GmurfProjectSaver: project_copy_files: Can't open source file."
		  " Returning -2");
	  return(-2);
	}

      strcpy(filedirname, dirname);
      if (filedirname[strlen(filedirname)] != '/')
	{
	  filedirname[strlen(filedirname)+1] = '\0';
	  filedirname[strlen(filedirname)] = '/';
	}

      strcat(filedirname,(tmpstr = remove_all_slashes(mixer_elt->tempgmurf->origname)));
      free(tmpstr);
      strcat(filedirname, "-");
      strcat(filedirname, filename);
      if ((dstfd = gzopen(filedirname, "wb")) < 0)
	{
	  warning("GmurfProjectSaver: project_copy_files: Can't open dest file."
		  "Returning -3");
	  return(-3);
	}

      strcpy(mixer_elt->tempgmurf->origname, filedirname);
      
      if (filename[2] < 'z')
	filename[2]++;
      else
	{
	  filename[1]++;
	  filename[2] = 'a';
	}
      if (filename[1] >= 'z')
	{
	  filename[1] = 'a';
	  filename[2] = 'a';
	  filename[0]++;
	}

      while((result = read(srcfd, buf, CPY_BUF_SIZE)) > 0)
	{
	  gzwrite(dstfd, buf, result);
	}
      close(srcfd);
      gzclose(dstfd);
      mixer_elt = mixer_elt->next_mixer_elt;
    }
  return(0);
}

int project_write_header(main_mixer, filename, gmurfrc)
     t_main_mixer* main_mixer;
     char*         filename;
     t_gmurfrc*    gmurfrc;

{
  int          i;
  t_mixer_elt* mixer_elt;
  t_point*     point;
  FILE*        fd;
  t_dsprc*     dsprc;
  
  fd = fopen(filename, "w");
  if (fd<0)
    {
      warning("GmurfProjectSaver.c: project_write_header: Can't open destination file. ");
      return(-1);
    }

  // Syntax information
  fprintf(fd, "%s", INFORMATION);
  mixer_elt = main_mixer->first_mixer_elt;
  while(mixer_elt != NULL)
    {

      // General information
      fprintf(fd, "%s", "\n######\n");
      fprintf(fd, "%s", ID_WAVE);
      fprintf(fd, "%s\n", mixer_elt->tempgmurf->origname);

      fprintf(fd, "%s", ID_VOL);
      fprintf(fd, "%d\n", (int) ((float)mixer_elt->mixer_info->volume*(float)DIV_VOLUME));

      fprintf(fd, "%s", ID_PAN);
      fprintf(fd, "%d\n", (int) ((float)mixer_elt->mixer_info->panning*(float)DIV_PANNING));

      fprintf(fd, "%s", ID_START);
      fprintf(fd, "%d\n", mixer_elt->mixer_info->begin_date);

      fprintf(fd, "%s", ID_DSP);
      dsprc = gmurfrc->first_dsp;
      i = 0;
      while ((dsprc != NULL) && (dsprc->dsp != mixer_elt->dsp))
	{
	  i++;
	  dsprc = dsprc->next_dsp;
	}
      if (dsprc == NULL)
	warning("GmurfProjectSaver: write_header: the t_mixer_elt*'s dsp "
		"was not found");
      fprintf(fd, "%d\n", i);

      if (mixer_elt->mixer_info->do_freq == TRUE_)
	{
	  fprintf(fd, "%s", ID_FRQON);
	}

      if (mixer_elt->mute == TRUE_)
	{
	  fprintf(fd, "%s", ID_MUTE);
	}

      // To be recorded ?
      if (mixer_elt->rec_or_play == REC)
	fprintf(fd, "%s", ID_REC);

      fprintf(fd, "%s", ID_ENDWAVE);

      // Volume curve
      point = mixer_elt->mixer_info->vol_curve;
      while(point != NULL)
	{
	  fprintf(fd, "%s", ID_VOLPOINT);
	  fprintf(fd, "%d\n%d\n", point->x, point->y);
	  point = point->next_point;
	}

      // Panning curve
      point = mixer_elt->mixer_info->pan_curve;
      while(point != NULL)
	{
	  fprintf(fd, "%s", ID_PANPOINT);
	  fprintf(fd, "%d\n%d\n", point->x, point->y);
	  point = point->next_point;
	}

      // Frequency curve
      point = mixer_elt->mixer_info->frq_curve;
      while(point != NULL)
	{
	  fprintf(fd, "%s", ID_FRQPOINT);
	  fprintf(fd, "%d\n%d\n", point->x, point->y);
	  point = point->next_point;
	}
      mixer_elt = mixer_elt->next_mixer_elt;
    }  
  // end of project
  fprintf(fd,"%s",ID_END);
  fclose(fd);
  return(0);
}

int project_save_file(main_mixer, dir_name, file_name, gmurfrc)
     t_main_mixer* main_mixer;
     char*         dir_name;
     char*         file_name;
     t_gmurfrc*    gmurfrc;
{
  char curdir[MAX_LINE_LEN];
  char filename[MAX_LINE_LEN];

  getcwd(curdir, MAX_LINE_LEN);

  strcpy(filename, file_name);
  if (chdir(dir_name)<0)
    {
      warning("GmurfProjectSaver.c: project_save_file: Couldn't "
	      "chdir. Returning -1");
      return(-1);
    }

  if (mkdir(filename,0755) < 0)
    {
      warning("GmurfProjectSaver: project_save_file: Can't create directory."
	      "Returning -1");
      return(-1);
    }

  /*  strcpy(filename, dir_name);
      if (filename[strlen(filename)] == '/')
      filename[strlen(filename)] = '\0';
  */

  if (project_copy_files(main_mixer, filename) != 0)
    return(-1);

  strcat(filename, GMURF_PROJECT_EXTENSION);

  if (project_write_header(main_mixer, filename,gmurfrc) != 0)
    return (-2);
  
  chdir(curdir);
  return(0);
}
