/*
** Renderer.c, for Gmurf
**
** Renders mixer to wave file. Seems to work well.
**
** sam-lo_p@epita.fr
*/

#include"Renderer.h"

// Saves mixer's old dsp configuration
t_dsp_save* mixer_mirror(main_mixer)
     t_main_mixer* main_mixer;
{
  t_dsp_save*  dsp_save;
  t_dsp_save*  first_dsp_save;
  t_mixer_elt* mixer_elt;

  mixer_elt = main_mixer->first_mixer_elt;
  
  if (mixer_elt != NULL)
    {
      first_dsp_save = (t_dsp_save*) malloc (sizeof(t_dsp_save));
      first_dsp_save->dsp = mixer_elt->dsp;
      
      dsp_save = first_dsp_save;

      while (mixer_elt->next_mixer_elt != NULL)
	{
	  mixer_elt = mixer_elt->next_mixer_elt;
	  dsp_save->next_dsp_save = (t_dsp_save*) malloc (sizeof(t_dsp_save));
	  dsp_save = dsp_save->next_dsp_save;
	  dsp_save->dsp = mixer_elt->dsp;
	} // while
      dsp_save->next_dsp_save = NULL;
    } // if

  else
    
    {
      warning("Renderer.c: mixer_mirror: I can't mirror a void main_mixer. "
	      "Returning NULL");
      return(NULL);
    }
  return(first_dsp_save);

}

// destroys the mixer dsps image
int destroy_mixer_mirror(dsp_save)
     t_dsp_save* dsp_save;
{
  t_dsp_save* next_dsp_save;

  if (dsp_save == NULL)
    {
      warning("Renderer.c: destroy_mixer_mirror: Trying to destroy a "
	      "NULL t_dsp_save. Returning -1");
      return(-1);
    }
  
  while(dsp_save != NULL)
    {
      next_dsp_save = dsp_save->next_dsp_save;
      free(dsp_save);
      dsp_save = next_dsp_save;
    }
  return(0);
}

// restores the mixer state
int restore_mixer_state(main_mixer, first_dsp_save)
     t_main_mixer* main_mixer;
     t_dsp_save*   first_dsp_save;
{
  t_mixer_elt* mixer_elt;
  t_dsp_save*  dsp_save;

  mixer_elt = main_mixer->first_mixer_elt;
  dsp_save  = first_dsp_save;

  while((mixer_elt != NULL) && (dsp_save != NULL))
    {
      mixer_elt->dsp = dsp_save->dsp;
      mixer_elt      = mixer_elt->next_mixer_elt;
      dsp_save       = dsp_save->next_dsp_save;
    }
  if ((mixer_elt != NULL) || (dsp_save != NULL))
    {
      warning("Renderer.c: restore_mixer_state: The numbers of t_mixer_elt* "
	      "and t_dsp_save* don't match.");
    }
  return(0);
}

// Replaces the mixer's dsps
int replace_mixer_dsps(main_mixer,dsp)
     t_main_mixer* main_mixer;
     t_dsp*        dsp;
{
  t_mixer_elt* mixer_elt;
  
  mixer_elt = main_mixer->first_mixer_elt;
  if (dsp == NULL)
    {
      warning("Renderer.c: replace_mixer_dsps: filling main_mixer with a "
	      "NULL t_dsp*");
    }
  while (mixer_elt != NULL)
    {
      mixer_elt->dsp = dsp;
      mixer_elt      = mixer_elt->next_mixer_elt;
    }
  return(0);
} 

char* lowcase(str)
     char* str;
{
  int i;
  
  i = 0;
  while (str[i] != '\0')
    {
      if ((str[i] >= 'A') && (str[i] <='Z'))
	str[i]+='a'-'A';
      i++;
    }
  return(str);
  
}

// Initializes the renderer
t_renderer* renderer_new(main_mixer, filename)
     t_main_mixer* main_mixer;
     char*         filename;
{
  t_dsp_save*   dsp_save;
  t_dsp*        dsp; 
  t_renderer*   renderer;
  int           fd;
  char          str[MAX_LINE_LEN];
  char* 	extension;
    
  // Testing if filename already has a WAV extension
  strcpy(str, filename);
  extension = (char*) malloc(sizeof(char)*4);
  if (strlen(filename) > 4)
    {
      strcpy(extension, filename+strlen(filename)-4);
      lowcase(extension);
      
      if (strcmp(extension, ".wav") != 0)
	strcat(str, ".wav");

      free(extension);
    }
  else
    {
      strcat(str, ".wav");
    }

  dsp_save         = mixer_mirror(main_mixer);
  if (dsp_save == NULL)
    return(NULL);
	
  fd = open(str, O_WRONLY | O_CREAT, 0644);

  if (fd<0)
    {
      warning("Renderer.c: renderer_new: I couldn't open the "
	      "destination file. Returning NULL");
      return(NULL);
    }

  lseek(fd, sizeof(t_wave_header), SEEK_SET);

  dsp              = (t_dsp*) malloc (sizeof(t_dsp));
  dsp->file        = fd;
  dsp->rec_or_play = PLAY;
  dsp->processed   = FALSE_;
  dsp->lock        = FALSE_;

  replace_mixer_dsps(main_mixer, dsp);

  renderer             = (t_renderer*) malloc(sizeof(t_renderer));
  renderer->dsp        = dsp;
  renderer->dsp_save   = dsp_save;
  renderer->main_mixer = main_mixer;

  mixer_seek_start(main_mixer);

  return(renderer);
}

// restore the mixer state and frees all
int renderer_destroy(renderer)
     t_renderer*   renderer;
{
  t_wave_header* waveheader;
  int            file_size;

  // porky has arrived on the earth. ;)
  file_size = lseek(renderer->dsp->file, 0, SEEK_END);

  // Writing the Microsoft WAV header
  waveheader = (t_wave_header*)malloc(sizeof(t_wave_header));

  strncpy(waveheader->riff_str, "RIFF",4);
  strncpy(waveheader->wave_fmt_str, "WAVEfmt ",8);
  strncpy(waveheader->data_str,"data",4);
  waveheader->smt_size         = 16;
  waveheader->format_tag       = 1;
  waveheader->chanels          = 2;
  waveheader->sample_rate      = DEF_RATE;
  waveheader->avg_byte_per_sec = DEF_RATE*4;
  waveheader->bits_per_sample  = DEF_BPS;
  waveheader->size             = file_size+44-8;
  waveheader->data_size        = file_size;
  waveheader->block_align      = 4;

  lseek(renderer->dsp->file, 0, SEEK_SET);
  write(renderer->dsp->file, waveheader, sizeof(t_wave_header));

  restore_mixer_state(renderer->main_mixer, renderer->dsp_save);
  close(renderer->dsp->file);
  free(renderer->dsp);
  free(renderer);
  free(waveheader);
  return(0);
}

// updates the renderer state
int renderer_update(renderer)
     t_renderer* renderer;
{
  return(play_buffers(renderer->main_mixer));
}

