/*
** Mixer.c for Gmurf in Gmurf/gui
** 
** Made by Rmi Coupet
** Login   <inthy@sietch.inthy.fr>
** 
** Started on  Thu Mar 11 16:39:28 1999 Rmi Coupet
** Last update Sun Mar 14 22:35:06 1999 Rmi Coupet
*/

#include "Mixer.h"
#include "xpms/pause_play.xpm"
#include "xpms/pause_play_light.xpm"
#include "xpms/play.xpm"
#include "xpms/play_light.xpm"
#include "xpms/stop_play.xpm"
#include "xpms/stop_play_light.xpm"

static void	mixer_class_init		(MixerClass	*klass) ;
static void	mixer_init			(Mixer		*ms) ;
void 		mixer_play			(GtkWidget	*widget) ;
void 		mixer_pause			(GtkWidget	*widget) ;
void 		mixer_stop			(GtkWidget	*widget) ;
void		mix_volume_change		(GtkWidget	*widget) ;

//static gint mixer_signals[LAST_SIGNAL] = { 0 };

guint mixer_get_type (void)
{
  static guint mg_type = 0;

  if (!mg_type)
  {
    GtkTypeInfo mg_info =
    {
      "Mixer",
      sizeof (Mixer),
      sizeof (MixerClass),
      (GtkClassInitFunc) mixer_class_init,
      (GtkObjectInitFunc) mixer_init,
      (GtkArgSetFunc) NULL,
//      (GtkArgGetFunc) NULL
    };

    mg_type = gtk_type_unique (gtk_vbox_get_type (), &mg_info);
  }
  
  return mg_type;
}

static void mixer_class_init (class)
MixerClass *class ;
{
  GtkObjectClass *object_class;
  
  object_class = (GtkObjectClass*) class;  
  class->mixer = NULL;
}

GtkWidget *mixer_new(void)
{
  return GTK_WIDGET ( gtk_type_new (mixer_get_type ()));
}

/* 
** The function which create the base of the Mixer object.
*/

static void	mixer_init (mg)
Mixer		*mg ;
{
  GtkWidget	*hbox ;
  GtkAdjustment	*adj ;

 
  mg->nb_ms = 0 ;
  mg->ms_list = NULL ;
  mg->vbox = gtk_vbox_new(FALSE, 1);
  mg->max_size = 0 ;
  mg->stop = PLAY_FLAG ;
  gtk_widget_show(mg->vbox) ;
 
  mg->frame = gtk_frame_new("GMURF   MIXER") ;
  gtk_frame_set_label_align(GTK_FRAME (mg->frame), 0.5, 0.0);
  gtk_widget_show(mg->frame) ;
  gtk_box_pack_start(GTK_BOX (mg), mg->frame, FALSE, FALSE, 0) ;

  mg->scrolled_window = gtk_scrolled_window_new(NULL, NULL) ;
  gtk_widget_set_name(mg->vbox, "scrolled window");
  gtk_widget_show(mg->scrolled_window) ;
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (mg->scrolled_window),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  //gtk_container_add(GTK_CONTAINER (mg->scrolled_window), mg->vbox);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW (mg->scrolled_window),
					mg->vbox) ;
  gtk_box_pack_start(GTK_BOX (mg), mg->scrolled_window, TRUE, TRUE, 0) ;  

  adj = (GtkAdjustment *) gtk_adjustment_new( 10, 0, 20, 1, 2, 1) ;
  mg->volume_hscale = gtk_hscale_new(adj) ;
  gtk_signal_connect_object(GTK_OBJECT (GTK_RANGE (mg->volume_hscale)->adjustment),
			    "value_changed",
			    mix_volume_change, GTK_OBJECT (mg)) ;
  gtk_widget_set_usize(mg->volume_hscale, 80, 30) ;
  gtk_widget_show(mg->volume_hscale) ;
  hbox = gtk_hbox_new(FALSE, 2) ;
  gtk_box_pack_start(GTK_BOX (hbox), mg->volume_hscale, FALSE, FALSE, 0) ;
  gtk_widget_show(hbox) ;

  mg->play_button = 
    Create_Button_with_pixmap_label("", play_xpm, 
				    play_light_xpm, MainWindow ) ;
  gtk_widget_show(mg->play_button) ;
  gtk_box_pack_start(GTK_BOX (hbox), mg->play_button, FALSE, FALSE, 0) ;

  gtk_signal_connect_object(GTK_OBJECT (mg->play_button), "clicked", 
			    GTK_SIGNAL_FUNC (mixer_play), GTK_OBJECT (mg)) ;

  mg->pause_button = 
    Create_Button_with_pixmap_label("", pause_play_xpm, 
				    pause_play_light_xpm, MainWindow ) ;
  gtk_widget_show(mg->pause_button) ;
  gtk_box_pack_start(GTK_BOX (hbox), mg->pause_button, FALSE, FALSE, 0) ;

  gtk_signal_connect_object(GTK_OBJECT (mg->pause_button), "clicked", 
			    GTK_SIGNAL_FUNC (mixer_pause), GTK_OBJECT (mg)) ;

  mg->stop_button = 
    Create_Button_with_pixmap_label("", stop_play_xpm, 
				    stop_play_light_xpm, MainWindow ) ;
  gtk_widget_show(mg->stop_button) ;
  gtk_box_pack_start(GTK_BOX (hbox), mg->stop_button, FALSE, FALSE, 0) ;

  gtk_signal_connect_object(GTK_OBJECT (mg->stop_button), "clicked", 
			    GTK_SIGNAL_FUNC (mixer_stop), GTK_OBJECT (mg)) ;

  gtk_box_pack_start(GTK_BOX (mg), hbox, FALSE, FALSE, 0) ;
  
  mg->progress_bar = gtk_progress_bar_new() ;
  gtk_widget_show(mg->progress_bar) ;
  gtk_box_pack_start(GTK_BOX (mg), mg->progress_bar, FALSE, FALSE, 0) ;
  mg->hscrollbar = gtk_hscrollbar_new(NULL) ;
  gtk_widget_show(mg->hscrollbar) ;
  gtk_widget_set_name(mg->hscrollbar, "hscroll");
  gtk_box_pack_start(GTK_BOX (mg), mg->hscrollbar, FALSE, FALSE, 0) ;
}


void 		mixer_play(widget)
GtkWidget	*widget ;
{
  Mixer		*mg ;
  t_ms_list	*aux ;

  mg = MIXER (widget) ;
  mixer_initialize(mg->Main_Mixer) ;
  
  if (mg->stop == PLAY_FLAG)
    mixer_seek_start(MIXER(Global_Mixer)->Main_Mixer);
  else
    mg->stop = PLAY_FLAG ;

  while ( (mixer_update_buffer(mg->Main_Mixer) != 0) && (mg->stop != STOP_FLAG) )
  {
    while(gtk_events_pending())
      gtk_main_iteration();
  }
  mixer_release(mg->Main_Mixer);
  if (mg->stop == PLAY_FLAG)
    mixer_seek_start(mg->Main_Mixer);
    
  aux = mg->ms_list ;
  while(aux != NULL)
  {
    if (aux->ms->elt->rec_or_play == REC)
      ms_draw_wav(aux->ms) ;
    aux = aux->next ;
  }
}

void 		mixer_pause(widget)
GtkWidget	*widget ;
{
  Mixer		*mg ;
  
  mg = MIXER (widget) ;
  mg->stop = STOP_FLAG ;
}


void 		mixer_stop(widget)
GtkWidget	*widget ;
{
  Mixer		*mg ;
  
  mg = MIXER (widget) ;
  mg->stop = STOP_FLAG ;
  mixer_seek_start(mg->Main_Mixer);
}


void		mix_volume_change(widget)
GtkWidget	*widget ;
{
  Mixer		*mg ;
  mg = MIXER (widget) ;
  
  mg->Main_Mixer->vol = (GTK_RANGE (mg->volume_hscale)->adjustment->value) / 10 ;
}






